==> scand-0.12.tar.gz: scand-0.12/scandctl.c <==
/* Copyright (c) 2003 Andrew Stikheev. All rights reserved.
Copyright (c) 2003 Andrew Stikheev. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: scandctl.c,v 1.7 2004/09/07 10:25:15 sand Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/mman.h>

#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include <err.h>
#include <errno.h>
#include <string.h>

#include "statistics.h"
#include "base.h"
#include "mem.h"

void *hashTable;

static int verbose 		= 0;
static systemParameters *sys;

static char *mmapedFile		= NULL;
static char ipAddr[16]		= ""; 
static int removeFlag		= 0;


static void PutOutStatistics	( int );
static void RemoveStatistics	();
static void ParseArgs		( int argc, char *argv[]  );
static void Usage		();
int background = 0;
////////////////////////////////////////////////////////////////////////////////
int main( int  argc, char *argv[] )
{
	int		fd;
	int 		fFlags;
	int 		mFlags;
 	struct stat 	filestat;

	ParseArgs( argc , argv );

	if ( !mmapedFile ) Usage();

	if ( removeFlag ) {

		fFlags = O_RDWR;
		mFlags = PROT_READ | PROT_WRITE;
	} else {
		fFlags = O_RDONLY;
		mFlags = PROT_READ;
	}

	fd = open ( mmapedFile, fFlags );
	if (fd < 0) Quit("open");
	if (fstat(fd,&filestat)) Quit("fstat");

	hashTable =  mmap(	0, 
		  		(size_t)filestat.st_size,
				mFlags,
				MAP_NOSYNC | MAP_SHARED,
				fd, 
				0  );

	if ( errno ) Quit("mmap");

	if ( removeFlag ) 	RemoveStatistics ();
	else			PutOutStatistics ( filestat.st_size );

	close( fd );

	return 0;
}
////////////////////////////////////////////////////////////////////////////////
/*
void Quit (const char* msg)             
{
        warn (msg);
        exit (1);
}
*/
////////////////////////////////////////////////////////////////////////////////
static void RemoveStatistics()
{
	cluster			*c;
	statisticItem		*item;
	int			i;
	int			j;
	in_addr_t		ipDelete = 0;
	int			notAll = 1;

	sys = (systemParameters *) hashTable;
	hashTable += sizeof(systemParameters) + sizeof(systemVariables);
	c = (cluster *) hashTable;

	if ( !strcasecmp ( ipAddr,"all" ) )  {
		notAll = 0;
	} else
	if ( (ipDelete=inet_addr(ipAddr)) == INADDR_NONE) {
		Quit("wrong ip-address");
	}

	for( i = 0 ; i < sys->numClusters; i++,c++) { 
		
		item  = hashTable + 
			sys->numClusters * sizeof(cluster) + 
			sizeof(statisticItem) * i * sys->clusterSize;

		for( j = 0 ; j < c->total; j++,item++ ) { 

			if ( notAll && item->srcIP != ipDelete )
				continue;
	
			item->srcIP = 0;
			if ( verbose ) printf("%s removed\n",ipAddr);
			if (notAll) return;
		}
	}
if (notAll) Quit("ip not found in table");
}
////////////////////////////////////////////////////////////////////////////////
static void PutOutStatistics(int memsize)
{
	cluster		*c;
	statisticItem	*item;
	connectItem	*con;
	struct in_addr	ip_src,ip_dst;
	int		j = 0;
	int		i;
	int		t,k;
	int 		show;
        int             show_all = 0;
	in_addr_t	ip = 0;
	void		*mem;
	systemVariables *sysV;
	char		*names[] = {"TCP","UDP","ICMP"};

        if (	strcasecmp(ipAddr,"all") && 
		strcmp(ipAddr,"") &&
		(ip=inet_addr(ipAddr)) == INADDR_NONE ) 
			Quit("wrong ip-address");

        if ( !strcasecmp(ipAddr,"all") ) show_all = 1;

	mem = malloc (  memsize );
	if ( !mem ) Quit ("malloc");

	memmove( mem, hashTable, memsize);

	hashTable = InitizializeMemory( mem, &sys, &sysV );

	printf("numClusters            %d\n",sys->numClusters);
	printf("clusterSize            %d\n",sys->clusterSize);
	printf("blockIfAny             %d\n",sys->blockIfAny);
	printf("enableBlock            %d\n",sys->enableBlock);
	printf("divertPort             %d\n",sys->divertPort);
	printf("mapFile                %s\n",sys->mapFile);
	printf("blkFile                %s\n",sys->blkFile);
	printf("blTFile                %s\n",sys->blTFile);
	printf("pidFile                %s\n",sys->pidFile);

	for(j = 0; j < NUM_TYPES; j++ ) 
		printf("maxConnects[%4s]      %d\n",names[j],
					 sys->maxConnects[j]);
	for(j = 0; j < NUM_TYPES; j++ )
		printf("minWeight[%4s]        %d\n",names[j],
					sys->minWeight[j]);

	for(j = 0; j < NUM_TYPES; j++ )
		printf("blkWeight[%4s]        %d\n",names[j],
					sys->blkWeight[j]);
        for(j = 0; j < NUM_TYPES; j++ )
                printf("maxWeight[%4s]        %d\n",names[j],
					sys->maxWeight[j]);

	for(j = 0; j < NUM_TYPES; j++ ) {
		printf("factor[%4s][ inHash]  %d\n",names[j], 
					sys->factor[j][1]);
		printf("factor[%4s][!inHash]  %d\n",names[j], 
					sys->factor[j][0]);
	}
	printf("\n");	
        printf("readPackets            %d\n",sysV->readPackets);
        printf("readPacketsFailure     %d\n",sysV->readPacketsFailure);
        printf("writePacketsFailure    %d\n",sysV->writePacketsFailure);
        printf("blockPackets           %d\n",sysV->blockPackets);
	printf("\n");	
	c = (cluster *)hashTable;
        for( j = 0; j < sys->numClusters; j++,c++) { 

		item = c->item;
              	for( i = 0; i < c->total; i++,item++ ) {

			if ( !item->srcIP || (ip && item->srcIP != ip ) ) 
				continue;

			if ( !ip ) {
				show = 0;
				for ( t = 0; t< NUM_TYPES; t++) {
					if ( item->weight[t] >= 
							sys->blkWeight[t] ) {
						show = 1;
						break;
					}
				}
				if ( !show_all && !show ) continue;
			}

                       	ip_src.s_addr = item->srcIP;
                       	printf( "%d[%d]  %s\n", j, i, inet_ntoa(ip_src));

			for ( t = 0; t < NUM_TYPES; t++) {

				printf("\t%4s:   %d\n",names[t],item->weight[t]);

				for ( k = 0; k < item->total[t]; k++ ) {

					con = item->con[t] + k;
                       			ip_dst.s_addr = con->dstIP;

					printf("\t\t%d -> %s:%d %d\n",
						ntohs(con->srcPort),
						inet_ntoa(ip_dst),
						ntohs(con->dstPort),
						(int)con->time );
				}
			}
               	}
  	}
	free(mem);
}
////////////////////////////////////////////////////////////////////////////////
static void ParseArgs(int argc, char *argv[])
{

	int 			ch;

	while( (ch=getopt(argc,argv,"vdm:i:")) != -1)

		switch (ch) {

		case 'm':
				mmapedFile = strdup (optarg);
				break;
		case 'i':
				if (strlen(optarg) > strlen("255.255.255.255")) 
					Usage();

				strcpy(ipAddr,optarg);
		 		break;
		case 'd': 	removeFlag = 1;
				break;
		case 'v':
				verbose = 1;
				break;
		default :
				Usage();
		}
}
////////////////////////////////////////////////////////////////////////////////
static void Usage()
{
	fprintf(stderr,"Usage: ./scandctl [ -d ip, all ] -m mapfile [ -i ip ]\n");
	exit(1);
}
////////////////////////////////////////////////////////////////////////////////

==> scand-0.14.tar.gz: scand-0.14/scandctl.c <==
/* Copyright (c) 2003 Andrew Stikheev. All rights reserved.
Copyright (c) 2003 Andrew Stikheev. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * $Id: scandctl.c,v 1.9 2006/03/09 02:09:47 leha Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/time.h>

#include <fcntl.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include <err.h>
#include <errno.h>
#include <string.h>

#include "statistics.h"
#include "base.h"
#include "mem.h"

void *hashTable;

static int verbose 		= 0;
static systemParameters *sys;

static char *mmapedFile		= NULL;
static char ipAddr[16]		= ""; 
static int removeFlag		= 0;


static void PutOutStatistics	( int );
static void RemoveStatistics	();
static void ParseArgs		( int argc, char *argv[]  );
static void Usage		();
int background = 0;
////////////////////////////////////////////////////////////////////////////////
int main( int  argc, char *argv[] )
{
	int		fd;
	int 		fFlags;
	int 		mFlags;
 	struct stat 	filestat;

	ParseArgs( argc , argv );

	if ( !mmapedFile ) Usage();

	if ( removeFlag ) {

		fFlags = O_RDWR;
		mFlags = PROT_READ | PROT_WRITE;
	} else {
		fFlags = O_RDONLY;
		mFlags = PROT_READ;
	}

	fd = open ( mmapedFile, fFlags );
	if (fd < 0) Quit("open");
	if (fstat(fd,&filestat)) Quit("fstat");

	hashTable =  mmap(	0, 
		  		(size_t)filestat.st_size,
				mFlags,
				MAP_NOSYNC | MAP_SHARED,
				fd, 
				0  );

	if ( errno ) Quit("mmap");

	if ( removeFlag ) 	RemoveStatistics ();
	else			PutOutStatistics ( filestat.st_size );

	close( fd );

	return 0;
}
////////////////////////////////////////////////////////////////////////////////
/*
void Quit (const char* msg)             
{
        warn (msg);
        exit (1);
}
*/
////////////////////////////////////////////////////////////////////////////////
static void RemoveStatistics()
{
	cluster			*c;
	statisticItem		*item;
	int			i;
	int			j;
	in_addr_t		ipDelete = 0;
	int			notAll = 1;

	sys = (systemParameters *) hashTable;
	hashTable += sizeof(systemParameters) + sizeof(systemVariables);
	c = (cluster *) hashTable;

	if ( !strcasecmp ( ipAddr,"all" ) )  {
		notAll = 0;
	} else
	if ( (ipDelete=inet_addr(ipAddr)) == INADDR_NONE) {
		Quit("wrong ip-address");
	}

	for( i = 0 ; i < sys->numClusters; i++,c++) { 
		
		item  = hashTable + 
			sys->numClusters * sizeof(cluster) + 
			sizeof(statisticItem) * i * sys->clusterSize;

		for( j = 0 ; j < c->total; j++,item++ ) { 

			if ( notAll && item->srcIP != ipDelete )
				continue;
	
			item->srcIP = 0;
			if ( verbose ) printf("%s removed\n",ipAddr);
			if (notAll) return;
		}
	}
if (notAll) Quit("ip not found in table");
}
////////////////////////////////////////////////////////////////////////////////
static void PutOutStatistics(int memsize)
{
	cluster		*c;
	statisticItem	*item;
	connectItem	*con;
	struct in_addr	ip_src,ip_dst;
	int		j = 0;
	static char 	stm[20];
	int		i;
	int		t,k;
	int 		show;
        int             show_all = 0;
	in_addr_t	ip = 0;
	void		*mem;
	systemVariables *sysV;
	char		*names[] = {"TCP","UDP","ICMP"};

        if (	strcasecmp(ipAddr,"all") && 
		strcmp(ipAddr,"") &&
		(ip=inet_addr(ipAddr)) == INADDR_NONE ) 
			Quit("wrong ip-address");
        if ( !strcasecmp(ipAddr,"all") ) show_all = 1;

	mem = malloc (  memsize );
	if ( !mem ) Quit ("malloc");

	memmove( mem, hashTable, memsize);

	hashTable = InitizializeMemory( mem, &sys, &sysV );

	printf("numClusters            %d\n",sys->numClusters);
	printf("clusterSize            %d\n",sys->clusterSize);
	printf("blockIfAny             %d\n",sys->blockIfAny);
	printf("enableBlock            %d\n",sys->enableBlock);
	printf("divertPort             %d\n",sys->divertPort);
	printf("mapFile                %s\n",sys->mapFile);
	printf("blkFile                %s\n",sys->blkFile);
	printf("blTFile                %s\n",sys->blTFile);
	printf("pidFile                %s\n",sys->pidFile);

	for(j = 0; j < NUM_TYPES; j++ ) 
		printf("maxConnects[%4s]      %d\n",names[j],
					 sys->maxConnects[j]);
	for(j = 0; j < NUM_TYPES; j++ )
		printf("minWeight[%4s]        %d\n",names[j],
					sys->minWeight[j]);

	for(j = 0; j < NUM_TYPES; j++ )
		printf("blkWeight[%4s]        %d\n",names[j],
					sys->blkWeight[j]);
        for(j = 0; j < NUM_TYPES; j++ )
                printf("maxWeight[%4s]        %d\n",names[j],
					sys->maxWeight[j]);

	for(j = 0; j < NUM_TYPES; j++ ) {
		printf("factor[%4s][ inHash]  %d\n",names[j], 
					sys->factor[j][1]);
		printf("factor[%4s][!inHash]  %d\n",names[j], 
					sys->factor[j][0]);
	}
	printf("\n");	
        printf("readPackets            %d\n",sysV->readPackets);
        printf("readPacketsFailure     %d\n",sysV->readPacketsFailure);
        printf("writePacketsFailure    %d\n",sysV->writePacketsFailure);
        printf("blockPackets           %d\n",sysV->blockPackets);
	printf("\n");	
	c = (cluster *)hashTable;
        for( j = 0; j < sys->numClusters; j++,c++) { 

		item = c->item;
              	for( i = 0; i < c->total; i++,item++ ) {

			if ( !item->srcIP || (ip && item->srcIP != ip ) ) 
				continue;

			if ( !ip ) {
				show = 0;
				for ( t = 0; t< NUM_TYPES; t++) {
					if ( item->weight[t] >= 
							sys->blkWeight[t] ) {
						show = 1;
						break;
					}
				}
				if ( !show_all && !show ) continue;
			}

                       	ip_src.s_addr = item->srcIP;
                       	printf( "%d[%d]  %s\n", j, i, inet_ntoa(ip_src));

			for ( t = 0; t < NUM_TYPES; t++) {

				printf("\t%4s:   %d\n",names[t],item->weight[t]);

				for ( k = 0; k < item->total[t]; k++ ) {

					con = item->con[t] + k;
					strftime(stm, sizeof(stm)-1, "%T",localtime(&con->time));
                       			ip_dst.s_addr = con->dstIP;
					printf("\t\t%d -> %s:%d %s\n",
						ntohs(con->srcPort),
						inet_ntoa(ip_dst),
						ntohs(con->dstPort),
						stm);
				}
			}
               	}
  	}
	free(mem);
}
////////////////////////////////////////////////////////////////////////////////
static void ParseArgs(int argc, char *argv[])
{

	int 			ch;

	while( (ch=getopt(argc,argv,"vdm:i:")) != -1)

		switch (ch) {

		case 'm':
				mmapedFile = strdup (optarg);
				break;
		case 'i':
				if (strlen(optarg) > strlen("255.255.255.255")) 
					Usage();

				strcpy(ipAddr,optarg);
		 		break;
		case 'd': 	removeFlag = 1;
				break;
		case 'v':
				verbose = 1;
				break;
		default :
				Usage();
		}
}
////////////////////////////////////////////////////////////////////////////////
static void Usage()
{
	fprintf(stderr,"Usage: ./scandctl [ -d ip, all ] -m mapfile [ -i ip ]\n");
	exit(1);
}
////////////////////////////////////////////////////////////////////////////////
